/* ************************************************
*  ASSESSING GLOBAL IMBALANCES IN A GVAR FRAMEWORK
*  -------------------------------------------------
*  Written by Yongcheol Shin. Last updated November 2007
*  Contact Y.SHIN@LEEDS.AC.UK
*  -------------------------------------------------
*  See Pesaran and Shin (1996, JoE, 1998, EL, 2003, ER),
*  GLPS (2003a, EJ, 2003b, JASA, 2006, OUP), PSW (2004), DdPS (2007),
*  Shin (2007) and GHS (2007)
*
*  This is a companion program of KGVARA.g and provides:
*
*  1. The confidence intervals of central forecasts 
*  2. Selected probability event forecasts
* 
*  Currently, we consider future uncertainty only based on the parametric 
*  stochastic bootstrap.
*
*  We plan to address parameter and model uncertainties in near future.
* ********************************************************************* */
NEW;  library pgraph;  graphset; time;

output file = kgvarB.out;

load BIGVAR1 BN GALL gb0 gb1 gd0 gd1 gdd gdt SIGE SIGU GX GVARp;
load nhorz sample case brpoint;
load Lhat A4Lhat ALhat Dhat A4Dhat ADhat;
load CHD header;

@ Some information necessary for running the program @
n_stoc = 1000; @ number of stochastic simulation @
/* nhorz = 24;    @ number of forecast horizon @ */
mg = 176;      @ number of global variables @

@ ************************************************************
* Out of Sample Probability Forecasts for inflation and growth
* ************************************************************
* E1. A single event: Pr(4-qt MA inflation < a% per annum)
*     where we use 15 threshold values:
*     a = (-2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)
*
* E2. A single event: Pr(4-qt output growth < a% per annum) 
*     where we use 12 threshold values:
*     a = (-2, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)
*
* E3. A single event of inflation target 1: Pr(a%<4-qt MA inflation<b%)
*
* E4. A single event of inflation target 2: Pr(4-qt MA inflation<c%)
*     e.g. between 1 and 3% or less than 2% for most developed countries
*
* E5. A single event of recession:
*     Pr(quarterly output growth < 0% for 2 consecutive quarters)
*
* E6. A single event of bad growth: Pr(4-qt MA output growths < d%)
*     e.g. less than the half of the historic average
*
* E7. Joint events of Pr(no recession and inflation target 1)
*
* E8. Joint events of Pr(good growth and inflation target 1)
*
* E9. Joint events of Pr(no recession and inflation target 2)
*
* E10. Joint events of Pr(good growth and inflation target 2)
* *********************************************************  @

@ *******************************************************
* Out of Sample Probability Forecasts for Current Account
* *******************************************************
* E1. A single event of CA surplus: Pr(4-qt MA CA > 0)
*
* E2. A single event of CA improvement: Pr(4-qt MA DCA > 0) 
*
* E3. A joint event of CA deficit and CA deterioration
*     Pr(4-qt MA CA < 0 & 4-qt MA DCA < 0)

* E4. A joint event of CA deficit and CA improvement 
*     Pr(4-qt MA CA < 0 & 4-qt MA DCA > 0)
*
* E5. A joint event of CA surplus and CA deterioration
*     Pr(4-qt MA CA > 0 & 4-qt MA DCA < 0)
*
* E6. A joint event of CA surplus and CA improvement 
*     Pr(4-qt MA CA > 0 & 4-qt MA DCA > 0)
* ****************************************  @

@ ************************** @
@ 1. future uncertainty only @
@ ************************** @
@* Singular value decomposition of SIGE @
{uu,ss,vv} = svd2(sige); 
uus = uu*sqrt(ss);

@ slection matrix @
if GVARp eq 1; JJ = eye(mg);
else;          JJ = eye(mg)~zeros(mg,mg*(GVARp-1));
endif;
   
@* Empirical distribution of forecasts: future uncertainty only @
nhm = nhorz*mg;  
SFL = zeros(nhm,n_stoc);   SFD = zeros(nhm,n_stoc);   @ quarterly rates @     
SA4FL = zeros(nhm,n_stoc); SA4FD = zeros(nhm,n_stoc); @ 4-qt MA @     
SAFL = zeros(nhm,n_stoc);  SAFD = zeros(nhm,n_stoc);  @ h-step avg @     

@* Probability forecasts: future uncertainty only @
A4DPUS = ZEROS(nhorz,n_stoc); DYUS = ZEROS(nhorz,n_stoc); A4DYUS = ZEROS(nhorz,n_stoc);
A4DPEU = ZEROS(nhorz,n_stoc); DYEU = ZEROS(nhorz,n_stoc); A4DYEU = ZEROS(nhorz,n_stoc);
A4DPJP = ZEROS(nhorz,n_stoc); DYJP = ZEROS(nhorz,n_stoc); A4DYJP = ZEROS(nhorz,n_stoc);
A4DPUK = ZEROS(nhorz,n_stoc); DYUK = ZEROS(nhorz,n_stoc); A4DYUK = ZEROS(nhorz,n_stoc);
A4DPNO = ZEROS(nhorz,n_stoc); DYNO = ZEROS(nhorz,n_stoc); A4DYNO = ZEROS(nhorz,n_stoc);
A4DPSD = ZEROS(nhorz,n_stoc); DYSD = ZEROS(nhorz,n_stoc); A4DYSD = ZEROS(nhorz,n_stoc);
A4DPSS = ZEROS(nhorz,n_stoc); DYSS = ZEROS(nhorz,n_stoc); A4DYSS = ZEROS(nhorz,n_stoc);
A4DPCA = ZEROS(nhorz,n_stoc); DYCA = ZEROS(nhorz,n_stoc); A4DYCA = ZEROS(nhorz,n_stoc);
A4DPAU = ZEROS(nhorz,n_stoc); DYAU = ZEROS(nhorz,n_stoc); A4DYAU = ZEROS(nhorz,n_stoc);
A4DPNZ = ZEROS(nhorz,n_stoc); DYNZ = ZEROS(nhorz,n_stoc); A4DYNZ = ZEROS(nhorz,n_stoc);
A4DPSF = ZEROS(nhorz,n_stoc); DYSF = ZEROS(nhorz,n_stoc); A4DYSF = ZEROS(nhorz,n_stoc);
A4DPAR = ZEROS(nhorz,n_stoc); DYAR = ZEROS(nhorz,n_stoc); A4DYAR = ZEROS(nhorz,n_stoc);
A4DPBR = ZEROS(nhorz,n_stoc); DYBR = ZEROS(nhorz,n_stoc); A4DYBR = ZEROS(nhorz,n_stoc);
A4DPCL = ZEROS(nhorz,n_stoc); DYCL = ZEROS(nhorz,n_stoc); A4DYCL = ZEROS(nhorz,n_stoc);
A4DPME = ZEROS(nhorz,n_stoc); DYME = ZEROS(nhorz,n_stoc); A4DYME = ZEROS(nhorz,n_stoc);
A4DPIN = ZEROS(nhorz,n_stoc); DYIN = ZEROS(nhorz,n_stoc); A4DYIN = ZEROS(nhorz,n_stoc);
A4DPKO = ZEROS(nhorz,n_stoc); DYKO = ZEROS(nhorz,n_stoc); A4DYKO = ZEROS(nhorz,n_stoc);
A4DPMA = ZEROS(nhorz,n_stoc); DYMA = ZEROS(nhorz,n_stoc); A4DYMA = ZEROS(nhorz,n_stoc);
A4DPPH = ZEROS(nhorz,n_stoc); DYPH = ZEROS(nhorz,n_stoc); A4DYPH = ZEROS(nhorz,n_stoc);
A4DPSI = ZEROS(nhorz,n_stoc); DYSI = ZEROS(nhorz,n_stoc); A4DYSI = ZEROS(nhorz,n_stoc);
A4DPTH = ZEROS(nhorz,n_stoc); DYTH = ZEROS(nhorz,n_stoc); A4DYTH = ZEROS(nhorz,n_stoc);
A4DPCN = ZEROS(nhorz,n_stoc); DYCN = ZEROS(nhorz,n_stoc); A4DYCN = ZEROS(nhorz,n_stoc);
A4DPID = ZEROS(nhorz,n_stoc); DYID = ZEROS(nhorz,n_stoc); A4DYID = ZEROS(nhorz,n_stoc);
A4DPPE = ZEROS(nhorz,n_stoc); DYPE = ZEROS(nhorz,n_stoc); A4DYPE = ZEROS(nhorz,n_stoc);
A4DPTU = ZEROS(nhorz,n_stoc); DYTU = ZEROS(nhorz,n_stoc); A4DYTU = ZEROS(nhorz,n_stoc);
A4DPSA = ZEROS(nhorz,n_stoc); DYSA = ZEROS(nhorz,n_stoc); A4DYSA = ZEROS(nhorz,n_stoc);

A4CAUS = ZEROS(nhorz,n_stoc); A4DCAUS = ZEROS(nhorz,n_stoc);
A4CAEU = ZEROS(nhorz,n_stoc); A4DCAEU = ZEROS(nhorz,n_stoc);
A4CAJP = ZEROS(nhorz,n_stoc); A4DCAJP = ZEROS(nhorz,n_stoc);
A4CAUK = ZEROS(nhorz,n_stoc); A4DCAUK = ZEROS(nhorz,n_stoc);
A4CANO = ZEROS(nhorz,n_stoc); A4DCANO = ZEROS(nhorz,n_stoc);
A4CASD = ZEROS(nhorz,n_stoc); A4DCASD = ZEROS(nhorz,n_stoc);
A4CASS = ZEROS(nhorz,n_stoc); A4DCASS = ZEROS(nhorz,n_stoc);
A4CAAU = ZEROS(nhorz,n_stoc); A4DCAAU = ZEROS(nhorz,n_stoc);
A4CACA = ZEROS(nhorz,n_stoc); A4DCACA = ZEROS(nhorz,n_stoc);
A4CAAU = ZEROS(nhorz,n_stoc); A4DCAAU = ZEROS(nhorz,n_stoc);
A4CANZ = ZEROS(nhorz,n_stoc); A4DCANZ = ZEROS(nhorz,n_stoc);
A4CASF = ZEROS(nhorz,n_stoc); A4DCASF = ZEROS(nhorz,n_stoc);
A4CAAR = ZEROS(nhorz,n_stoc); A4DCAAR = ZEROS(nhorz,n_stoc);
A4CABR = ZEROS(nhorz,n_stoc); A4DCABR = ZEROS(nhorz,n_stoc);
A4CACL = ZEROS(nhorz,n_stoc); A4DCACL = ZEROS(nhorz,n_stoc);
A4CAME = ZEROS(nhorz,n_stoc); A4DCAME = ZEROS(nhorz,n_stoc);
A4CAIN = ZEROS(nhorz,n_stoc); A4DCAIN = ZEROS(nhorz,n_stoc);
A4CAKO = ZEROS(nhorz,n_stoc); A4DCAKO = ZEROS(nhorz,n_stoc);
A4CAMA = ZEROS(nhorz,n_stoc); A4DCAMA = ZEROS(nhorz,n_stoc);
A4CAPH = ZEROS(nhorz,n_stoc); A4DCAPH = ZEROS(nhorz,n_stoc);
A4CASI = ZEROS(nhorz,n_stoc); A4DCASI = ZEROS(nhorz,n_stoc);
A4CATH = ZEROS(nhorz,n_stoc); A4DCATH = ZEROS(nhorz,n_stoc);
A4CACN = ZEROS(nhorz,n_stoc); A4DCACN = ZEROS(nhorz,n_stoc);
A4CAID = ZEROS(nhorz,n_stoc); A4DCAID = ZEROS(nhorz,n_stoc);
A4CAPE = ZEROS(nhorz,n_stoc); A4DCAPE = ZEROS(nhorz,n_stoc);
A4CATU = ZEROS(nhorz,n_stoc); A4DCATU = ZEROS(nhorz,n_stoc);
A4CASA = ZEROS(nhorz,n_stoc); A4DCASA = ZEROS(nhorz,n_stoc);

rndseed 2222;
sj = 1;
do until sj > n_stoc;
output off; "replication" sj;

@ Drawing the mg by nhorz vector of N(0,1) for future errors which
* are perfectly symmetric with zero mean by construction @

/* 
   if     sj eq 1; rtmp = rndn(mg,nhorz); rtmp0 = rtmp;
   elseif sj eq 2; rtmp = -rtmp;          rtmp1 = rtmp;
   else;
      if rtmp1 eq -rtmp0; rtmp = rndn(mg,nhorz);
      else; rtmp = -rtmp;
      endif;
      rtmp0 = rtmp1; rtmp1 = rtmp; @ updating @
   endif;
*/
   rtmp = rndn(mg,nhorz);
   rtmp = uus*rtmp; @ errors with the same covariance @

@ simulation of future forecast errors @
   vrtmp = vec(rtmp); @ mg*nh by 1 @
   rhat = zeros(mg,nhorz); tem1 = eye(mg*GVARp);
   zi = 1;
   do until zi > nhorz;
      tem1 = BIGVAR1*tem1;
      if zi eq 1; tem3 = eye(mg); BJ = 0;
      else;       tem3 = BJ~tem3;
      endif;

      if     zi < nhorz;  tem3a = tem3~zeros(mg,mg*(nhorz-zi));
      elseif zi eq nhorz; tem3a = tem3;
      endif;

      Bj = JJ*tem1*JJ';
      rhat[.,zi] = tem3a*vrtmp;
      zi = zi + 1;
   endo;

@ level forecasts plus simulated errors @
   SLhat = Lhat + rhat'; @ mg by nh @

@ Forecasts of 4-qt MA, average: level @
  { SA4Lhat, SALhat } = Elevel(SLhat,GX,nhorz,mg);

@ Forecasts of quarterly, annual average, average: FD @
  { SDhat, SA4Dhat, Sadhat } = Echange(Slhat,GX,nhorz,mg);

@ Saving 4-qt MA inflation and growth forecasts for individual countries
  for probility forecasts of 10 events: each 1 by nh vector @

   A4DPUS[.,sj]= SA4Dhat[.,6];     DYUS[.,sj]= SDhat[.,7];     A4DYUS[.,sj]= SA4Dhat[.,7];

   A4DPEU[.,sj]= SA4Lhat[.,7+6];   DYEU[.,sj]= SDhat[.,7+7];   A4DYEU[.,sj]= SA4Dhat[.,7+7];
   A4DPJP[.,sj]= SA4Lhat[.,2*7+6]; DYJP[.,sj]= SDhat[.,2*7+7]; A4DYJP[.,sj]= SA4Dhat[.,2*7+7];
   A4DPUK[.,sj]= SA4Lhat[.,3*7+6]; DYUK[.,sj]= SDhat[.,3*7+7]; A4DYUK[.,sj]= SA4Dhat[.,3*7+7];
   A4DPNO[.,sj]= SA4Lhat[.,4*7+6]; DYNO[.,sj]= SDhat[.,4*7+7]; A4DYNO[.,sj]= SA4Dhat[.,4*7+7];
   A4DPSD[.,sj]= SA4Lhat[.,5*7+6]; DYSD[.,sj]= SDhat[.,5*7+7]; A4DYSD[.,sj]= SA4Dhat[.,5*7+7];
   A4DPSS[.,sj]= SA4Lhat[.,6*7+6]; DYSS[.,sj]= SDhat[.,6*7+7]; A4DYSS[.,sj]= SA4Dhat[.,6*7+7];
   A4DPCA[.,sj]= SA4Lhat[.,7*7+6]; DYCA[.,sj]= SDhat[.,7*7+7]; A4DYCA[.,sj]= SA4Dhat[.,7*7+7];
   A4DPAU[.,sj]= SA4Lhat[.,8*7+6]; DYAU[.,sj]= SDhat[.,8*7+7]; A4DYAU[.,sj]= SA4Dhat[.,8*7+7];
   A4DPNZ[.,sj]= SA4Lhat[.,9*7+6]; DYNZ[.,sj]= SDhat[.,9*7+7]; A4DYNZ[.,sj]= SA4Dhat[.,9*7+7];
   A4DPSF[.,sj]= SA4Lhat[.,10*7+6]; DYSF[.,sj]= SDhat[.,10*7+7]; A4DYSF[.,sj]= SA4Dhat[.,10*7+7];
   A4DPAR[.,sj]= SA4Lhat[.,11*7+6]; DYAR[.,sj]= SDhat[.,11*7+7]; A4DYAR[.,sj]= SA4Dhat[.,11*7+7];
   A4DPBR[.,sj]= SA4Lhat[.,12*7+6]; DYBR[.,sj]= SDhat[.,12*7+7]; A4DYBR[.,sj]= SA4Dhat[.,12*7+7];
   A4DPCL[.,sj]= SA4Lhat[.,13*7+6]; DYCL[.,sj]= SDhat[.,13*7+7]; A4DYCL[.,sj]= SA4Dhat[.,13*7+7];
   A4DPME[.,sj]= SA4Lhat[.,14*7+6]; DYME[.,sj]= SDhat[.,14*7+7]; A4DYME[.,sj]= SA4Dhat[.,14*7+7];
   A4DPIN[.,sj]= SA4Lhat[.,15*7+6]; DYIN[.,sj]= SDhat[.,15*7+7]; A4DYIN[.,sj]= SA4Dhat[.,15*7+7];
   A4DPKO[.,sj]= SA4Lhat[.,16*7+6]; DYKO[.,sj]= SDhat[.,16*7+7]; A4DYKO[.,sj]= SA4Dhat[.,16*7+7];
   A4DPMA[.,sj]= SA4Lhat[.,17*7+6]; DYMA[.,sj]= SDhat[.,17*7+7]; A4DYMA[.,sj]= SA4Dhat[.,17*7+7];
   A4DPPH[.,sj]= SA4Lhat[.,18*7+6]; DYPH[.,sj]= SDhat[.,18*7+7]; A4DYPH[.,sj]= SA4Dhat[.,18*7+7];
   A4DPSI[.,sj]= SA4Lhat[.,19*7+6]; DYSI[.,sj]= SDhat[.,19*7+7]; A4DYSI[.,sj]= SA4Dhat[.,19*7+7];
   A4DPTH[.,sj]= SA4Lhat[.,20*7+6]; DYTH[.,sj]= SDhat[.,20*7+7]; A4DYTH[.,sj]= SA4Dhat[.,20*7+7];
   A4DPCN[.,sj]= SA4Lhat[.,21*7+5]; DYCN[.,sj]= SDhat[.,21*7+6]; A4DYCN[.,sj]= SA4Dhat[.,21*7+6];
   A4DPID[.,sj]= SA4Lhat[.,21*7+6+5]; DYID[.,sj]= SDhat[.,21*7+6+6]; A4DYID[.,sj]= SA4Dhat[.,21*7+6+6];
   A4DPPE[.,sj]= SA4Lhat[.,21*7+2*6+5]; DYPE[.,sj]= SDhat[.,21*7+2*6+6]; A4DYPE[.,sj]= SA4Dhat[.,21*7+2*6+6];
   A4DPTU[.,sj]= SA4Lhat[.,21*7+3*6+5]; DYTU[.,sj]= SDhat[.,21*7+3*6+6]; A4DYTU[.,sj]= SA4Dhat[.,21*7+3*6+6];
   A4DPSA[.,sj]= SA4Lhat[.,21*7+4*6+4]; DYSA[.,sj]= SDhat[.,21*7+4*6+5]; A4DYSA[.,sj]= SA4Dhat[.,21*7+4*6+5];

@ Saving 4-qt Current account and its difference for individual countries
  for probility forecasts of 4 joint events: each 1 by nh vector @

   A4CAUS[.,sj]= SA4Lhat[.,4]-SA4Lhat[.,3];         A4DCAUS[.,sj]= SA4Dhat[.,4]-SA4Dhat[.,3];
   A4CAEU[.,sj]= SA4Lhat[.,7+4]-SA4Lhat[.,7+3];     A4DCAEU[.,sj]= SA4Dhat[.,7+4]-SA4Dhat[.,7+3];
   A4CAJP[.,sj]= SA4Lhat[.,2*7+4]-SA4Lhat[.,2*7+3]; A4DCAJP[.,sj]= SA4Dhat[.,2*7+4]-SA4Dhat[.,2*7+3];
   A4CAUK[.,sj]= SA4Lhat[.,3*7+4]-SA4Lhat[.,3*7+3]; A4DCAUK[.,sj]= SA4Dhat[.,3*7+4]-SA4Dhat[.,3*7+3];
   A4CANO[.,sj]= SA4Lhat[.,4*7+4]-SA4Lhat[.,4*7+3]; A4DCANO[.,sj]= SA4Dhat[.,4*7+4]-SA4Dhat[.,4*7+3];
   A4CASD[.,sj]= SA4Lhat[.,5*7+4]-SA4Lhat[.,5*7+3]; A4DCASD[.,sj]= SA4Dhat[.,5*7+4]-SA4Dhat[.,5*7+3];
   A4CASS[.,sj]= SA4Lhat[.,6*7+4]-SA4Lhat[.,6*7+3]; A4DCASS[.,sj]= SA4Dhat[.,6*7+4]-SA4Dhat[.,6*7+3];
   A4CACA[.,sj]= SA4Lhat[.,7*7+4]-SA4Lhat[.,7*7+3]; A4DCACA[.,sj]= SA4Dhat[.,7*7+4]-SA4Dhat[.,7*7+3];
   A4CAAU[.,sj]= SA4Lhat[.,8*7+4]-SA4Lhat[.,8*7+3]; A4DCAAU[.,sj]= SA4Dhat[.,8*7+4]-SA4Dhat[.,8*7+3];
   A4CANZ[.,sj]= SA4Lhat[.,9*7+4]-SA4Lhat[.,9*7+3]; A4DCANZ[.,sj]= SA4Dhat[.,9*7+4]-SA4Dhat[.,9*7+3];
   A4CASF[.,sj]= SA4Lhat[.,10*7+4]-SA4Lhat[.,10*7+3]; A4DCASF[.,sj]= SA4Dhat[.,10*7+4]-SA4Dhat[.,10*7+3];
   A4CAAR[.,sj]= SA4Lhat[.,11*7+4]-SA4Lhat[.,11*7+3]; A4DCAAR[.,sj]= SA4Dhat[.,11*7+4]-SA4Dhat[.,11*7+3];
   A4CABR[.,sj]= SA4Lhat[.,12*7+4]-SA4Lhat[.,12*7+3]; A4DCABR[.,sj]= SA4Dhat[.,12*7+4]-SA4Dhat[.,12*7+3];
   A4CACL[.,sj]= SA4Lhat[.,13*7+4]-SA4Lhat[.,13*7+3]; A4DCACL[.,sj]= SA4Dhat[.,13*7+4]-SA4Dhat[.,13*7+3];
   A4CAME[.,sj]= SA4Lhat[.,14*7+4]-SA4Lhat[.,14*7+3]; A4DCAME[.,sj]= SA4Dhat[.,14*7+4]-SA4Dhat[.,14*7+3];
   A4CAIN[.,sj]= SA4Lhat[.,15*7+4]-SA4Lhat[.,15*7+3]; A4DCAIN[.,sj]= SA4Dhat[.,15*7+4]-SA4Dhat[.,15*7+3];
   A4CAKO[.,sj]= SA4Lhat[.,16*7+4]-SA4Lhat[.,16*7+3]; A4DCAKO[.,sj]= SA4Dhat[.,16*7+4]-SA4Dhat[.,16*7+3];
   A4CAMA[.,sj]= SA4Lhat[.,17*7+4]-SA4Lhat[.,17*7+3]; A4DCAMA[.,sj]= SA4Dhat[.,17*7+4]-SA4Dhat[.,17*7+3];
   A4CAPH[.,sj]= SA4Lhat[.,18*7+4]-SA4Lhat[.,18*7+3]; A4DCAPH[.,sj]= SA4Dhat[.,18*7+4]-SA4Dhat[.,18*7+3];
   A4CASI[.,sj]= SA4Lhat[.,19*7+4]-SA4Lhat[.,19*7+3]; A4DCASI[.,sj]= SA4Dhat[.,19*7+4]-SA4Dhat[.,19*7+3];
   A4CATH[.,sj]= SA4Lhat[.,20*7+4]-SA4Lhat[.,20*7+3]; A4DCATH[.,sj]= SA4Dhat[.,20*7+4]-SA4Dhat[.,20*7+3];
   A4CACN[.,sj]= SA4Lhat[.,21*7+4]-SA4Lhat[.,21*7+3]; A4DCACN[.,sj]= SA4Dhat[.,21*7+4]-SA4Dhat[.,21*7+3];
   A4CAID[.,sj]= SA4Lhat[.,21*7+6+4]-SA4Lhat[.,21*7+6+3]; A4DCAID[.,sj]= SA4Dhat[.,21*7+6+4]-SA4Dhat[.,21*7+6+3];
   A4CAPE[.,sj]= SA4Lhat[.,21*7+2*6+4]-SA4Lhat[.,21*7+2*6+3]; A4DCAPE[.,sj]= SA4Dhat[.,21*7+2*6+4]-SA4Dhat[.,21*7+2*6+3];
   A4CATU[.,sj]= SA4Lhat[.,21*7+3*6+4]-SA4Lhat[.,21*7+3*6+3]; A4DCATU[.,sj]= SA4Dhat[.,21*7+3*6+4]-SA4Dhat[.,21*7+3*6+3];
   A4CASA[.,sj]= SA4Lhat[.,21*7+4*6+3]-SA4Lhat[.,21*7+4*6+2]; A4DCASA[.,sj]= SA4Dhat[.,21*7+4*6+3]-SA4Dhat[.,21*7+4*6+2];

@ vectorising & saving all: nh*mg by 1 vector @
   SFL[.,sj] = vec(SLhat); SFD[.,sj] = vec(SDhat); 
   SA4FL[.,sj] = vec(SA4Lhat); SA4FD[.,sj] = vec(SA4Dhat); 
   SAFL[.,sj] = vec(SALhat); SAFD[.,sj] = vec(SADhat); 

   sj = sj + 1;
endo;

save SFL SFD SA4FL SA4FD SAFL SAFD;

@ ******************************************************* @
@ Probility event forecasts of 10 events for each country @
@ ******************************************************* @

DGX = GX-lag(GX); DGX = DGX[2:rows(DGX),.]; ndgx = rows(dgx);

@ output growth at T @
DYTUS = DGX[ndgx,7];
DYTEU = DGX[ndgx,7+7];
DYTJP = DGX[ndgx,2*7+7];
DYTUK = DGX[ndgx,3*7+7];
DYTNO = DGX[ndgx,4*7+7];
DYTSD = DGX[ndgx,5*7+7];
DYTSS = DGX[ndgx,6*7+7];
DYTCA = DGX[ndgx,7*7+7];
DYTAU = DGX[ndgx,8*7+7];
DYTNZ = DGX[ndgx,9*7+7];
DYTSF = DGX[ndgx,10*7+7];
DYTAR = DGX[ndgx,11*7+7];
DYTBR = DGX[ndgx,12*7+7];
DYTCL = DGX[ndgx,13*7+7];
DYTME = DGX[ndgx,14*7+7];
DYTIN = DGX[ndgx,15*7+7];
DYTKO = DGX[ndgx,16*7+7];
DYTMA = DGX[ndgx,17*7+7];
DYTPH = DGX[ndgx,18*7+7];
DYTSI = DGX[ndgx,19*7+7];
DYTTH = DGX[ndgx,20*7+7];
DYTCN = DGX[ndgx,21*7+6];
DYTID = DGX[ndgx,21*7+6+6];
DYTPE = DGX[ndgx,21*7+2*6+6];
DYTTU = DGX[ndgx,21*7+3*6+6];
DYTSA = DGX[ndgx,21*7+4*6+5];

@ average inflation over the sample period, annual %, may change period @
AVGDPUS = 400*meanc(DGX[.,6]);
AVGDPEU = 400*meanc(GX[.,7+6]);
AVGDPJP = 400*meanc(GX[.,2*7+6]);
AVGDPUK = 400*meanc(GX[.,3*7+6]);
AVGDPNO = 400*meanc(GX[.,4*7+6]);
AVGDPSD = 400*meanc(GX[.,5*7+6]);
AVGDPSS = 400*meanc(GX[.,6*7+6]);
AVGDPCA = 400*meanc(GX[.,7*7+6]);
AVGDPAU = 400*meanc(GX[.,8*7+6]);
AVGDPNZ = 400*meanc(GX[.,9*7+6]);
AVGDPSF = 400*meanc(GX[.,10*7+6]);
AVGDPAR = 400*meanc(GX[.,11*7+6]);
AVGDPBR = 400*meanc(GX[.,12*7+6]);
AVGDPCL = 400*meanc(GX[.,13*7+6]);
AVGDPME = 400*meanc(GX[.,14*7+6]);
AVGDPIN = 400*meanc(GX[.,15*7+6]);
AVGDPKO = 400*meanc(GX[.,16*7+6]);
AVGDPMA = 400*meanc(GX[.,17*7+6]);
AVGDPPH = 400*meanc(GX[.,18*7+6]);
AVGDPSI = 400*meanc(GX[.,19*7+6]);
AVGDPTH = 400*meanc(GX[.,20*7+6]);
AVGDPCN = 400*meanc(GX[.,21*7+5]);
AVGDPID = 400*meanc(GX[.,21*7+6+5]);
AVGDPPE = 400*meanc(GX[.,21*7+2*6+5]);
AVGDPTU = 400*meanc(GX[.,21*7+3*6+5]);
AVGDPSA = 400*meanc(GX[.,21*7+4*6+4]);

@ average growth rate over the sample period, annual %, may change period @
AVGGRUS = 400*meanc(DGX[.,7]);
AVGGREU = 400*meanc(DGX[.,7+7]);
AVGGRJP = 400*meanc(DGX[.,2*7+7]);
AVGGRUK = 400*meanc(DGX[.,3*7+7]);
AVGGRNO = 400*meanc(DGX[.,4*7+7]);
AVGGRSD = 400*meanc(DGX[.,5*7+7]);
AVGGRSS = 400*meanc(DGX[.,6*7+7]);
AVGGRCA = 400*meanc(DGX[.,7*7+7]);
AVGGRAU = 400*meanc(DGX[.,8*7+7]);
AVGGRNZ = 400*meanc(DGX[.,9*7+7]);
AVGGRSF = 400*meanc(DGX[.,10*7+7]);
AVGGRAR = 400*meanc(DGX[.,11*7+7]);
AVGGRBR = 400*meanc(DGX[.,12*7+7]);
AVGGRCL = 400*meanc(DGX[.,13*7+7]);
AVGGRME = 400*meanc(DGX[.,14*7+7]);
AVGGRIN = 400*meanc(DGX[.,15*7+7]);
AVGGRKO = 400*meanc(DGX[.,16*7+7]);
AVGGRMA = 400*meanc(DGX[.,17*7+7]);
AVGGRPH = 400*meanc(DGX[.,18*7+7]);
AVGGRSI = 400*meanc(DGX[.,19*7+7]);
AVGGRTH = 400*meanc(DGX[.,20*7+7]);
AVGGRCN = 400*meanc(DGX[.,21*7+6]);
AVGGRID = 400*meanc(DGX[.,21*7+6+6]);
AVGGRPE = 400*meanc(DGX[.,21*7+2*6+6]);
AVGGRTU = 400*meanc(DGX[.,21*7+3*6+6]);
AVGGRSA = 400*meanc(DGX[.,21*7+4*6+5]);

@ printing @
format /rd 6,2; output on; 
"average inflation and growth over the sample period"; 
avgus = avgdpus~avggrus; avgus;
avgeu = avgdpeu~avggreu; avgeu;
avgjp = avgdpjp~avggrjp; avgjp;
avguk = avgdpuk~avggruk; avguk;
avgno = avgdpno~avggrno; avgno;
avgsd = avgdpsd~avggrsd; avgsd;
avgss = avgdpss~avggrss; avgss;
avgca = avgdpca~avggrca; avgca;
avgau = avgdpau~avggrau; avgau;
avgnz = avgdpnz~avggrnz; avgnz;
avgar = avgdpar~avggrar; avgar;
avgbr = avgdpbr~avggrbr; avgbr;
avgcl = avgdpcl~avggrcl; avgcl;
avgme = avgdpme~avggrme; avgme;
avgin = avgdpin~avggrin; avgin;
avgko = avgdpko~avggrko; avgko;
avgma = avgdpma~avggrma; avgma;
avgph = avgdpph~avggrph; avgph;
avgsi = avgdpsi~avggrsi; avgsi;
avgth = avgdpth~avggrth; avgth;
avgcn = avgdpcn~avggrcn; avgcn;
avgid = avgdpid~avggrid; avgid;
avgpe = avgdppe~avggrpe; avgpe;
avgtu = avgdptu~avggrtu; avgtu;
avgsa = avgdpsa~avggrsa; avgsa;

@ current account  @
CAUS = GX[.,4] - GX[.,3];
CAEU = GX[.,7+4] - GX[.,7+3];
CAJP = GX[.,2*7+4] - GX[.,2*7+3];
CAUK = GX[.,3*7+4] - GX[.,3*7+3];
CANO = GX[.,4*7+4] - GX[.,4*7+3];
CASD = GX[.,5*7+4] - GX[.,5*7+3];
CASS = GX[.,6*7+4] - GX[.,6*7+3];
CACA = GX[.,7*7+4] - GX[.,7*7+3];
CAAU = GX[.,8*7+4] - GX[.,8*7+3];
CANZ = GX[.,9*7+4] - GX[.,9*7+3];
CASF = GX[.,10*7+4] - GX[.,10*7+3];
CAAR = GX[.,11*7+4] - GX[.,11*7+3];
CABR = GX[.,12*7+4] - GX[.,12*7+3];
CACL = GX[.,13*7+4] - GX[.,13*7+3];
CAME = GX[.,14*7+4] - GX[.,14*7+3];
CAIN = GX[.,15*7+4] - GX[.,15*7+3];
CAKO = GX[.,16*7+4] - GX[.,16*7+3];
CAMA = GX[.,17*7+4] - GX[.,17*7+3];
CAPH = GX[.,18*7+4] - GX[.,18*7+3];
CASI = GX[.,19*7+4] - GX[.,19*7+3];
CATH = GX[.,20*7+4] - GX[.,20*7+3];
CACN = GX[.,21*7+4] - GX[.,7+3];
CAID = GX[.,21*7+6+4] - GX[.,21*7+6+3];
CAPE = GX[.,21*7+2*6+4] - GX[.,21*7+2*6+3];
CATU = GX[.,21*7+3*6+4] - GX[.,21*7+3*6+3];
CASA = GX[.,21*7+4*6+3] - GX[.,21*7+4*6+2];

@ changes in current account  @
nca = rows(CAUS);
DCAUS = CAUS[2:nca]-CAUS[1:nca-1]; 
DCAEU = CAEU[2:nca]-CAEU[1:nca-1];
DCAJP = CAJP[2:nca]-CAJP[1:nca-1];
DCAUK = CAUK[2:nca]-CAUK[1:nca-1];
DCANO = CANO[2:nca]-CANO[1:nca-1];
DCASD = CASD[2:nca]-CASD[1:nca-1];
DCASS = CASS[2:nca]-CASS[1:nca-1];
DCACA = CACA[2:nca]-CACA[1:nca-1];
DCAAU = CAAU[2:nca]-CAAU[1:nca-1];
DCANZ = CANZ[2:nca]-CANZ[1:nca-1];
DCASF = CASF[2:nca]-CASF[1:nca-1];
DCAAR = CAAR[2:nca]-CAAR[1:nca-1];
DCABR = CABR[2:nca]-CABR[1:nca-1];
DCACL = CACL[2:nca]-CACL[1:nca-1];
DCAME = CAME[2:nca]-CAME[1:nca-1];
DCAID = CAID[2:nca]-CAID[1:nca-1];
DCAKO = CAKO[2:nca]-CAKO[1:nca-1];
DCAMA = CAMA[2:nca]-CAMA[1:nca-1];
DCAPH = CAPH[2:nca]-CAPH[1:nca-1];
DCASI = CASI[2:nca]-CASI[1:nca-1];
DCATH = CATH[2:nca]-CATH[1:nca-1];
DCACN = CACN[2:nca]-CACN[1:nca-1];
DCAID = CAID[2:nca]-CAID[1:nca-1];
DCAPE = CAPE[2:nca]-CAPE[1:nca-1];
DCATU = CATU[2:nca]-CATU[1:nca-1];
DCASA = CASA[2:nca]-CASA[1:nca-1];

@ thresholds (% per annum) for inflation and output for CDF @

THDPUS = seqa(-2,1,15); @ -2% to 13% @ 
THDYUS = seqa(-2,1,15); @ -2% to 13% @
THDPEU = seqa(-2,1,15); THDYEU = seqa(-2,1,15); 
THDPJP = seqa(-2,1,15); THDYJP = seqa(-2,1,15); 
THDPUK = seqa(-2,1,15); THDYUK = seqa(-2,1,15); 
THDPNO = seqa(-2,1,15); THDYNO = seqa(-2,1,15); 
THDPSD = seqa(-2,1,15); THDYSD = seqa(-2,1,15); 
THDPSS = seqa(-2,1,15); THDYSS = seqa(-2,1,15); 
THDPCA = seqa(-2,1,15); THDYCA = seqa(-2,1,15); 
THDPAU = seqa(-2,1,15); THDYAU = seqa(-2,1,15); 
THDPNZ = seqa(-2,1,15); THDYNZ = seqa(-2,1,15); 
THDPSF = seqa(-2,1,15); THDYSF = seqa(-2,1,15); 
THDPAR = seqa(-2,1,15); THDYAR = seqa(-2,1,15); 
THDPBR = seqa(-2,1,15); THDYBR = seqa(-2,1,15); 
THDPCL = seqa(-2,1,15); THDYCL = seqa(-2,1,15); 
THDPME = seqa(-2,1,15); THDYME = seqa(-2,1,15); 
THDPIN = seqa(-2,1,15); THDYIN = seqa(-2,1,15); 
THDPKO = seqa(-2,1,15); THDYKO = seqa(-2,1,15); 
THDPMA = seqa(-2,1,15); THDYMA = seqa(-2,1,15); 
THDPPH = seqa(-2,1,15); THDYPH = seqa(-2,1,15); 
THDPSI = seqa(-2,1,15); THDYSI = seqa(-2,1,15); 
THDPTH = seqa(-2,1,15); THDYTH = seqa(-2,1,15); 
THDPCN = seqa(-2,1,15); THDYCN = seqa(-2,1,15); 
THDPID = seqa(-2,1,15); THDYID = seqa(-2,1,15); 
THDPPE = seqa(-2,1,15); THDYPE = seqa(-2,1,15); 
THDPTU = seqa(-2,1,15); THDYTU = seqa(-2,1,15); 
THDPSA = seqa(-2,1,15); THDYSA = seqa(-2,1,15); 

@ Inflation taregts (lower and upper intervals) and the single target @ 
@ if none, use the 1-3% band for most developed countries @

EVDPUS = zeros(3,1); @ NONE @
EVDPUS[1] = 1; EVDPUS[2] = 3; EVDPUS[3] = 2; @ between 1 and 3% and <2% @ 
EVDPEU = zeros(3,1); @ HICP close to 2% @
EVDPEU[1] = 1; EVDPEU[2] = 3; EVDPEU[3] = 2; @ between 1 and 3% and <2% @ 
EVDPJP = zeros(3,1); @ None @
EVDPJP[1] = 1; EVDPJP[2] = 3; EVDPJP[3] = 2; @ between 1 and 3% and <2% @ 
EVDPUK = zeros(3,1); @ CPI 2% @
EVDPUK[1] = 1; EVDPUK[2] = 3; EVDPUK[3] = 2; @ between 1 and 3% and <2% @ 
EVDPNO = zeros(3,1); @ CPI 2.5% @
EVDPNO[1] = 1.5; EVDPNO[2] = 3.5; EVDPNO[3] = 2.5; @ between 1.5 and 3.5% and <2.5% @ 
EVDPSD = zeros(3,1); @ CPI 2% @
EVDPSD[1] = 1; EVDPSD[2] = 3; EVDPSD[3] = 2; @ between 1 and 3% and <2% @ 
EVDPSS = zeros(3,1); @ CPI <2% @
EVDPSS[1] = 1; EVDPSS[2] = 3; EVDPSS[3] = 2; @ between 1 and 3% and <2% @ 
EVDPCA = zeros(3,1); @ CPI 2% @
EVDPCA[1] = 1; EVDPCA[2] = 3; EVDPCA[3] = 2; @ between 1 and 3% and <2% @ 
EVDPAU = zeros(3,1); @ CPI 2-3% @
EVDPAU[1] = 1.5; EVDPAU[2] = 3.5; EVDPAU[3] = 2.5; @ between 1.5 and 3.5% and <2.5% @ 
EVDPNZ = zeros(3,1); @ CPI 1-3% @
EVDPNZ[1] = 1; EVDPNZ[2] = 3; EVDPNZ[3] = 2; @ between 1 and 3% and <2% @ 
EVDPSF = zeros(3,1); @ CPIX 3-6% @
EVDPSF[1] = 3; EVDPSF[2] = 6; EVDPSF[3] = 4.5; @ between 3 and 6% and <4.5% @ 
EVDPAR = zeros(3,1); @ NONE, USE BR @
EVDPAR[1] = 2.5; EVDPAR[2] = 6.5; EVDPAR[3] = 4.5; @ between 2.5 and 6.5% and <4.5% @ 
EVDPBR = zeros(3,1); @ 4.5% CPI WITH 2% BAND @
EVDPBR[1] = 2.5; EVDPBR[2] = 6.5; EVDPBR[3] = 4.5; @ between 2.5 and 6.5% and <4.5% @ 
EVDPCL = zeros(3,1); @ CPI 3% @
EVDPCL[1] = 2; EVDPCL[2] = 4; EVDPCL[3] = 3; @ between 2 and 4% and <3% @ 
EVDPME = zeros(3,1); @ CPI 3% @
EVDPME[1] = 2; EVDPME[2] = 4; EVDPME[3] = 3; @ between 2 and 4% and <3% @ 
EVDPIN = zeros(3,1); @ NONE @
EVDPIN[1] = 1; EVDPIN[2] = 3; EVDPIN[3] = 2; @ between 1 and 3% and <2% @ 
EVDPKO = zeros(3,1); @ 2.5-3.5% @
EVDPKO[1] = 2; EVDPKO[2] = 4; EVDPKO[3] = 3; @ between 2 and 4% and <3% @ 
EVDPMA = zeros(3,1); @ NONE @
EVDPMA[1] = 1; EVDPMA[2] = 3; EVDPMA[3] = 2; @ between 1 and 3% and <2% @ 
EVDPPH = zeros(3,1); @ 4-5% @
EVDPPH[1] = 3.5; EVDPPH[2] = 5.5; EVDPPH[3] = 4.5; @ between 3.5 and 5.5% and <4.5% @ 
EVDPSI = zeros(3,1); @ NONE @
EVDPSI[1] = 1; EVDPSI[2] = 3; EVDPSI[3] = 2; @ between 1 and 3% and <2% @ 
EVDPTH = zeros(3,1); @ 0-3.5% @
EVDPTH[1] = 2.5; EVDPTH[2] = 4.5; EVDPTH[3] = 3.5; @ between 2.5 and 4.5% and <3.5% @ 
EVDPCN = zeros(3,1); @ NONE @
EVDPCN[1] = 1; EVDPCN[2] = 3; EVDPCN[3] = 2; @ between 1 and 3% and <2% @ 
EVDPID = zeros(3,1); @ 5% at 2007, 3% at lt @
EVDPID[1] = 4; EVDPID[2] = 6; EVDPID[3] = 5; @ between 4 and 6% and <5% @ 
EVDPPE = zeros(3,1); @ 1.5-3.5% BEFORE FEB 2007, then 1-3% @
EVDPPE[1] = 1.5; EVDPPE[2] = 3.5; EVDPPE[3] = 2.5; @ between 1.5 and 3.5% and <2.5% @ 
EVDPTU = zeros(3,1); @ 5% with 2% band, only at IT cince 2006 @
EVDPTU[1] = 3; EVDPTU[2] = 7; EVDPTU[3] = 5; @ between 4 and 6% and <5% @ 
EVDPSA = zeros(3,1); @ NONE @
EVDPSA[1] = 1; EVDPSA[2] = 3; EVDPSA[3] = 2; @ between 1 and 3% and <2% @ 

@ Growth: recession and the low growth @ 

EVDYUS = zeros(2,1); EVDYUS[1] = 0; EVDYUS[2] = AVGGRUS/2;
EVDYEU = zeros(2,1); EVDYEU[1] = 0; EVDYEU[2] = AVGGREU/2;        
EVDYJP = zeros(2,1); EVDYJP[1] = 0; EVDYJP[2] = AVGGRJP/2;        
EVDYUK = zeros(2,1); EVDYUK[1] = 0; EVDYUK[2] = AVGGRUK/2;        
EVDYNO = zeros(2,1); EVDYNO[1] = 0; EVDYNO[2] = AVGGRNO/2;        
EVDYSD = zeros(2,1); EVDYSD[1] = 0; EVDYSD[2] = AVGGRSD/2;       
EVDYSS = zeros(2,1); EVDYSS[1] = 0; EVDYSS[2] = AVGGRSS/2;       
EVDYCA = zeros(2,1); EVDYCA[1] = 0; EVDYCA[2] = AVGGRCA/2;       
EVDYAU = zeros(2,1); EVDYAU[1] = 0; EVDYAU[2] = AVGGRAU/2;       
EVDYNZ = zeros(2,1); EVDYNZ[1] = 0; EVDYNZ[2] = AVGGRNZ/2;       
EVDYSF = zeros(2,1); EVDYSF[1] = 0; EVDYSF[2] = AVGGRSF/2;       
EVDYAR = zeros(2,1); EVDYAR[1] = 0; EVDYAR[2] = AVGGRAR/2;       
EVDYBR = zeros(2,1); EVDYBR[1] = 0; EVDYBR[2] = AVGGRBR/2;       
EVDYCL = zeros(2,1); EVDYCL[1] = 0; EVDYCL[2] = AVGGRCL/2;       
EVDYME = zeros(2,1); EVDYME[1] = 0; EVDYME[2] = AVGGRME/2;       
EVDYIN = zeros(2,1); EVDYIN[1] = 0; EVDYIN[2] = AVGGRIN/2;       
EVDYKO = zeros(2,1); EVDYKO[1] = 0; EVDYKO[2] = AVGGRKO/2;       
EVDYMA = zeros(2,1); EVDYMA[1] = 0; EVDYMA[2] = AVGGRMA/2;       
EVDYPH = zeros(2,1); EVDYPH[1] = 0; EVDYPH[2] = AVGGRPH/2;       
EVDYSI = zeros(2,1); EVDYSI[1] = 0; EVDYSI[2] = AVGGRSI/2;       
EVDYTH = zeros(2,1); EVDYTH[1] = 0; EVDYTH[2] = AVGGRTH/2;       
EVDYCN = zeros(2,1); EVDYCN[1] = 0; EVDYCN[2] = AVGGRCN/2;       
EVDYID = zeros(2,1); EVDYID[1] = 0; EVDYID[2] = AVGGRID/2;       
EVDYPE = zeros(2,1); EVDYPE[1] = 0; EVDYPE[2] = AVGGRPE/2;       
EVDYTU = zeros(2,1); EVDYTU[1] = 0; EVDYTU[2] = AVGGRTU/2;       
EVDYSA = zeros(2,1); EVDYSA[1] = 0; EVDYSA[2] = AVGGRSA/2;       

@ evaluation of prob event forecasting for each countries @

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPUS,DYUS,A4DYUS,DYTUS,THDPUS,THDYUS,EVDPUS,EVDYUS);
oo1 = xlswritem(SPE1,"SPRUS","b2",1,"");
oo2 = xlswritem(SPE2,"SPRUS","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRUS","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPEU,DYEU,A4DYEU,DYTEU,THDPEU,THDYEU,EVDPEU,EVDYEU);
oo1 = xlswritem(SPE1,"SPREU","b2",1,"");
oo2 = xlswritem(SPE2,"SPREU","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPREU","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPJP,DYJP,A4DYJP,DYTJP,THDPJP,THDYJP,EVDPJP,EVDYJP);
oo1 = xlswritem(SPE1,"SPRJP","b2",1,"");
oo2 = xlswritem(SPE2,"SPRJP","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRJP","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPUK,DYUK,A4DYUK,DYTUK,THDPUK,THDYUK,EVDPUK,EVDYUK);
oo1 = xlswritem(SPE1,"SPRUK","b2",1,"");
oo2 = xlswritem(SPE2,"SPRUK","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRUK","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPNO,DYNO,A4DYNO,DYTNO,THDPNO,THDYNO,EVDPNO,EVDYNO);
oo1 = xlswritem(SPE1,"SPRNO","b2",1,"");
oo2 = xlswritem(SPE2,"SPRNO","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRNO","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPSD,DYSD,A4DYSD,DYTSD,THDPSD,THDYSD,EVDPSD,EVDYSD);
oo1 = xlswritem(SPE1,"SPRSD","b2",1,"");
oo2 = xlswritem(SPE2,"SPRSD","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRSD","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPSS,DYSS,A4DYSS,DYTSS,THDPSS,THDYSS,EVDPSS,EVDYSS);
oo1 = xlswritem(SPE1,"SPRSS","b2",1,"");
oo2 = xlswritem(SPE2,"SPRSS","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRSS","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPCA,DYCA,A4DYCA,DYTCA,THDPCA,THDYCA,EVDPCA,EVDYCA);
oo1 = xlswritem(SPE1,"SPRCA","b2",1,"");
oo2 = xlswritem(SPE2,"SPRCA","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRCA","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPAU,DYAU,A4DYAU,DYTAU,THDPAU,THDYAU,EVDPAU,EVDYAU);
oo1 = xlswritem(SPE1,"SPRAU","b2",1,"");
oo2 = xlswritem(SPE2,"SPRAU","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRAU","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPNZ,DYNZ,A4DYNZ,DYTNZ,THDPNZ,THDYNZ,EVDPNZ,EVDYNZ);
oo1 = xlswritem(SPE1,"SPRNZ","b2",1,"");
oo2 = xlswritem(SPE2,"SPRNZ","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRNZ","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPSF,DYSF,A4DYSF,DYTSF,THDPSF,THDYSF,EVDPSF,EVDYSF);
oo1 = xlswritem(SPE1,"SPRSF","b2",1,"");
oo2 = xlswritem(SPE2,"SPRSF","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRSF","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPAR,DYAR,A4DYAR,DYTAR,THDPAR,THDYAR,EVDPAR,EVDYAR);
oo1 = xlswritem(SPE1,"SPRAR","b2",1,"");
oo2 = xlswritem(SPE2,"SPRAR","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRAR","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPBR,DYBR,A4DYBR,DYTBR,THDPBR,THDYBR,EVDPBR,EVDYBR);
oo1 = xlswritem(SPE1,"SPRBR","b2",1,"");
oo2 = xlswritem(SPE2,"SPRBR","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRBR","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPCL,DYCL,A4DYCL,DYTCL,THDPCL,THDYCL,EVDPCL,EVDYCL);
oo1 = xlswritem(SPE1,"SPRCL","b2",1,"");
oo2 = xlswritem(SPE2,"SPRCL","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRCL","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPME,DYME,A4DYME,DYTME,THDPME,THDYME,EVDPME,EVDYME);
oo1 = xlswritem(SPE1,"SPRME","b2",1,"");
oo2 = xlswritem(SPE2,"SPRME","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRME","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPIN,DYIN,A4DYIN,DYTIN,THDPIN,THDYIN,EVDPIN,EVDYIN);
oo1 = xlswritem(SPE1,"SPRIN","b2",1,"");
oo2 = xlswritem(SPE2,"SPRIN","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRIN","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPKO,DYKO,A4DYKO,DYTKO,THDPKO,THDYKO,EVDPKO,EVDYKO);
oo1 = xlswritem(SPE1,"SPRKO","b2",1,"");
oo2 = xlswritem(SPE2,"SPRKO","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRKO","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPMA,DYMA,A4DYMA,DYTMA,THDPMA,THDYMA,EVDPMA,EVDYMA);
oo1 = xlswritem(SPE1,"SPRMA","b2",1,"");
oo2 = xlswritem(SPE2,"SPRMA","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRMA","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPPH,DYPH,A4DYPH,DYTPH,THDPPH,THDYPH,EVDPPH,EVDYPH);
oo1 = xlswritem(SPE1,"SPRPH","b2",1,"");
oo2 = xlswritem(SPE2,"SPRPH","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRPH","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPSI,DYSI,A4DYSI,DYTSI,THDPSI,THDYSI,EVDPSI,EVDYSI);
oo1 = xlswritem(SPE1,"SPRSI","b2",1,"");
oo2 = xlswritem(SPE2,"SPRSI","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRSI","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPTH,DYTH,A4DYTH,DYTTH,THDPTH,THDYTH,EVDPTH,EVDYTH);
oo1 = xlswritem(SPE1,"SPRTH","b2",1,"");
oo2 = xlswritem(SPE2,"SPRTH","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRTH","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPCN,DYCN,A4DYCN,DYTCN,THDPCN,THDYCN,EVDPCN,EVDYCN);
oo1 = xlswritem(SPE1,"SPRCN","b2",1,"");
oo2 = xlswritem(SPE2,"SPRCN","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRCN","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPID,DYID,A4DYID,DYTID,THDPID,THDYID,EVDPID,EVDYID);
oo1 = xlswritem(SPE1,"SPRID","b2",1,"");
oo2 = xlswritem(SPE2,"SPRID","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRID","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPPE,DYPE,A4DYPE,DYTPE,THDPPE,THDYPE,EVDPPE,EVDYPE);
oo1 = xlswritem(SPE1,"SPRPE","b2",1,"");
oo2 = xlswritem(SPE2,"SPRPE","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRPE","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPTU,DYTU,A4DYTU,DYTTU,THDPTU,THDYTU,EVDPTU,EVDYTU);
oo1 = xlswritem(SPE1,"SPRTU","b2",1,"");
oo2 = xlswritem(SPE2,"SPRTU","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRTU","b2",3,"");

{ SPE1, SPE2, SPE3, SPE4, SPE5, SPE6, SPE7, SPE8, SPE9, SPE10 }
= PREVENT(A4DPSA,DYSA,A4DYSA,DYTSA,THDPSA,THDYSA,EVDPSA,EVDYSA);
oo1 = xlswritem(SPE1,"SPRSA","b2",1,"");
oo2 = xlswritem(SPE2,"SPRSA","b2",2,"");
SPEOUT = SPE3~SPE4~SPE5~SPE6~SPE7~SPE8~SPE9~SPE10;
oo3 = xlswritem(SPEOUT,"SPRSA","b2",3,"");

@ CA events @

{ CAPR } = PREVCA(A4CAUS,A4DCAUS);
oc1 = xlswritem(CAPR,"SPRUS","b2",4,"");

{ CAPR } = PREVCA(A4CAEU,A4DCAEU);
oc1 = xlswritem(CAPR,"SPREU","b2",4,"");

{ CAPR } = PREVCA(A4CAJP,A4DCAJP);
oc1 = xlswritem(CAPR,"SPRJP","b2",4,"");

{ CAPR } = PREVCA(A4CAUK,A4DCAUK);
oc1 = xlswritem(CAPR,"SPRUK","b2",4,"");

{ CAPR } = PREVCA(A4CANO,A4DCANO);
oc1 = xlswritem(CAPR,"SPRNO","b2",4,"");

{ CAPR } = PREVCA(A4CASD,A4DCASD);
oc1 = xlswritem(CAPR,"SPRSD","b2",4,"");

{ CAPR } = PREVCA(A4CASS,A4DCASS);
oc1 = xlswritem(CAPR,"SPRSS","b2",4,"");

{ CAPR } = PREVCA(A4CACA,A4DCACA);
oc1 = xlswritem(CAPR,"SPRCA","b2",4,"");

{ CAPR } = PREVCA(A4CAAU,A4DCAAU);
oc1 = xlswritem(CAPR,"SPRAU","b2",4,"");

{ CAPR } = PREVCA(A4CANZ,A4DCANZ);
oc1 = xlswritem(CAPR,"SPRNZ","b2",4,"");

{ CAPR } = PREVCA(A4CASF,A4DCASF);
oc1 = xlswritem(CAPR,"SPRSF","b2",4,"");

{ CAPR } = PREVCA(A4CAAR,A4DCAAR);
oc1 = xlswritem(CAPR,"SPRAR","b2",4,"");

{ CAPR } = PREVCA(A4CABR,A4DCABR);
oc1 = xlswritem(CAPR,"SPRBR","b2",4,"");

{ CAPR } = PREVCA(A4CACL,A4DCACL);
oc1 = xlswritem(CAPR,"SPRCL","b2",4,"");

{ CAPR } = PREVCA(A4CAME,A4DCAME);
oc1 = xlswritem(CAPR,"SPRME","b2",4,"");

{ CAPR } = PREVCA(A4CAIN,A4DCAIN);
oc1 = xlswritem(CAPR,"SPRIN","b2",4,"");

{ CAPR } = PREVCA(A4CAKO,A4DCAKO);
oc1 = xlswritem(CAPR,"SPRKO","b2",4,"");

{ CAPR } = PREVCA(A4CAMA,A4DCAMA);
oc1 = xlswritem(CAPR,"SPRMA","b2",4,"");

{ CAPR } = PREVCA(A4CAPH,A4DCAPH);
oc1 = xlswritem(CAPR,"SPRPH","b2",4,"");

{ CAPR } = PREVCA(A4CASI,A4DCASI);
oc1 = xlswritem(CAPR,"SPRSI","b2",4,"");

{ CAPR } = PREVCA(A4CATH,A4DCATH);
oc1 = xlswritem(CAPR,"SPRTH","b2",4,"");

{ CAPR } = PREVCA(A4CACN,A4DCACN);
oc1 = xlswritem(CAPR,"SPRCN","b2",4,"");

{ CAPR } = PREVCA(A4CAID,A4DCAID);
oc1 = xlswritem(CAPR,"SPRID","b2",4,"");

{ CAPR } = PREVCA(A4CAPE,A4DCAPE);
oc1 = xlswritem(CAPR,"SPRPE","b2",4,"");

{ CAPR } = PREVCA(A4CATU,A4DCATU);
oc1 = xlswritem(CAPR,"SPRTU","b2",4,"");

{ CAPR } = PREVCA(A4CASA,A4DCASA);
oc1 = xlswritem(CAPR,"SPRSA","b2",4,"");

@ HEADERS @
nn1 = {"E1 DP" "-2" "-1" "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13"};
nn2 = {"E2 DY" "-2" "-1" "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" "10" "11" "12" "13"};
nn3 = {"E3" "E4" "E5" "E6" "E7" "E8" "E9" "E10"};
nn4 = {"E1" "E2" "E3" "E4" "E5" "E6"};

on1 = xlswritesa(nn1,"SPRUS","a1",1,"");
on2 = xlswritesa(nn2,"SPRUS","a1",2,"");
on3 = xlswritesa(nn3,"SPRUS","b1",3,"");
on4 = xlswritesa(nn4,"SPRUS","b1",4,"");

on1 = xlswritesa(nn1,"SPREU","a1",1,"");
on2 = xlswritesa(nn2,"SPREU","a1",2,"");
on3 = xlswritesa(nn3,"SPREU","b1",3,"");
on4 = xlswritesa(nn4,"SPREU","b1",4,"");

on1 = xlswritesa(nn1,"SPRJP","a1",1,"");
on2 = xlswritesa(nn2,"SPRJP","a1",2,"");
on3 = xlswritesa(nn3,"SPRJP","b1",3,"");
on4 = xlswritesa(nn4,"SPRJP","b1",4,"");

on1 = xlswritesa(nn1,"SPRUK","a1",1,"");
on2 = xlswritesa(nn2,"SPRUK","a1",2,"");
on3 = xlswritesa(nn3,"SPRUK","b1",3,"");
on4 = xlswritesa(nn4,"SPRUK","b1",4,"");

on1 = xlswritesa(nn1,"SPRNO","a1",1,"");
on2 = xlswritesa(nn2,"SPRNO","a1",2,"");
on3 = xlswritesa(nn3,"SPRNO","b1",3,"");
on4 = xlswritesa(nn4,"SPRNO","b1",4,"");

on1 = xlswritesa(nn1,"SPRSD","a1",1,"");
on2 = xlswritesa(nn2,"SPRSD","a1",2,"");
on3 = xlswritesa(nn3,"SPRSD","b1",3,"");
on4 = xlswritesa(nn4,"SPRSD","b1",4,"");

on1 = xlswritesa(nn1,"SPRSS","a1",1,"");
on2 = xlswritesa(nn2,"SPRSS","a1",2,"");
on3 = xlswritesa(nn3,"SPRSS","b1",3,"");
on4 = xlswritesa(nn4,"SPRSS","b1",4,"");

on1 = xlswritesa(nn1,"SPRCA","a1",1,"");
on2 = xlswritesa(nn2,"SPRCA","a1",2,"");
on3 = xlswritesa(nn3,"SPRCA","b1",3,"");
on4 = xlswritesa(nn4,"SPRCA","b1",4,"");

on1 = xlswritesa(nn1,"SPRAU","a1",1,"");
on2 = xlswritesa(nn2,"SPRAU","a1",2,"");
on3 = xlswritesa(nn3,"SPRAU","b1",3,"");
on4 = xlswritesa(nn4,"SPRAU","b1",4,"");

on1 = xlswritesa(nn1,"SPRNZ","a1",1,"");
on2 = xlswritesa(nn2,"SPRNZ","a1",2,"");
on3 = xlswritesa(nn3,"SPRNZ","b1",3,"");
on4 = xlswritesa(nn4,"SPRNZ","b1",4,"");

on1 = xlswritesa(nn1,"SPRSF","a1",1,"");
on2 = xlswritesa(nn2,"SPRSF","a1",2,"");
on3 = xlswritesa(nn3,"SPRSF","b1",3,"");
on4 = xlswritesa(nn4,"SPRSF","b1",4,"");

on1 = xlswritesa(nn1,"SPRAR","a1",1,"");
on2 = xlswritesa(nn2,"SPRAR","a1",2,"");
on3 = xlswritesa(nn3,"SPRAR","b1",3,"");
on4 = xlswritesa(nn4,"SPRAR","b1",4,"");

on1 = xlswritesa(nn1,"SPRBR","a1",1,"");
on2 = xlswritesa(nn2,"SPRBR","a1",2,"");
on3 = xlswritesa(nn3,"SPRBR","b1",3,"");
on4 = xlswritesa(nn4,"SPRBR","b1",4,"");

on1 = xlswritesa(nn1,"SPRCL","a1",1,"");
on2 = xlswritesa(nn2,"SPRCL","a1",2,"");
on3 = xlswritesa(nn3,"SPRCL","b1",3,"");
on4 = xlswritesa(nn4,"SPRCL","b1",4,"");

on1 = xlswritesa(nn1,"SPRME","a1",1,"");
on2 = xlswritesa(nn2,"SPRME","a1",2,"");
on3 = xlswritesa(nn3,"SPRME","b1",3,"");
on4 = xlswritesa(nn4,"SPRME","b1",4,"");

on1 = xlswritesa(nn1,"SPRIN","a1",1,"");
on2 = xlswritesa(nn2,"SPRIN","a1",2,"");
on3 = xlswritesa(nn3,"SPRIN","b1",3,"");
on4 = xlswritesa(nn4,"SPRIN","b1",4,"");

on1 = xlswritesa(nn1,"SPRKO","a1",1,"");
on2 = xlswritesa(nn2,"SPRKO","a1",2,"");
on3 = xlswritesa(nn3,"SPRKO","b1",3,"");
on4 = xlswritesa(nn4,"SPRKO","b1",4,"");

on1 = xlswritesa(nn1,"SPRMA","a1",1,"");
on2 = xlswritesa(nn2,"SPRMA","a1",2,"");
on3 = xlswritesa(nn3,"SPRMA","b1",3,"");
on4 = xlswritesa(nn4,"SPRMA","b1",4,"");

on1 = xlswritesa(nn1,"SPRPH","a1",1,"");
on2 = xlswritesa(nn2,"SPRPH","a1",2,"");
on3 = xlswritesa(nn3,"SPRPH","b1",3,"");
on4 = xlswritesa(nn4,"SPRPH","b1",4,"");

on1 = xlswritesa(nn1,"SPRSI","a1",1,"");
on2 = xlswritesa(nn2,"SPRSI","a1",2,"");
on3 = xlswritesa(nn3,"SPRSI","b1",3,"");
on4 = xlswritesa(nn4,"SPRSI","b1",4,"");

on1 = xlswritesa(nn1,"SPRTH","a1",1,"");
on2 = xlswritesa(nn2,"SPRTH","a1",2,"");
on3 = xlswritesa(nn3,"SPRTH","b1",3,"");
on4 = xlswritesa(nn4,"SPRTH","b1",4,"");

on1 = xlswritesa(nn1,"SPRCN","a1",1,"");
on2 = xlswritesa(nn2,"SPRCN","a1",2,"");
on3 = xlswritesa(nn3,"SPRCN","b1",3,"");
on4 = xlswritesa(nn4,"SPRCN","b1",4,"");

on1 = xlswritesa(nn1,"SPRID","a1",1,"");
on2 = xlswritesa(nn2,"SPRID","a1",2,"");
on3 = xlswritesa(nn3,"SPRID","b1",3,"");
on4 = xlswritesa(nn4,"SPRID","b1",4,"");

on1 = xlswritesa(nn1,"SPRPE","a1",1,"");
on2 = xlswritesa(nn2,"SPRPE","a1",2,"");
on3 = xlswritesa(nn3,"SPRPE","b1",3,"");
on4 = xlswritesa(nn4,"SPRPE","b1",4,"");

on1 = xlswritesa(nn1,"SPRTU","a1",1,"");
on2 = xlswritesa(nn2,"SPRTU","a1",2,"");
on3 = xlswritesa(nn3,"SPRTU","b1",3,"");
on4 = xlswritesa(nn4,"SPRTU","b1",4,"");

on1 = xlswritesa(nn1,"SPRSA","a1",1,"");
on2 = xlswritesa(nn2,"SPRSA","a1",2,"");
on3 = xlswritesa(nn3,"SPRSA","b1",3,"");
on4 = xlswritesa(nn4,"SPRSA","b1",4,"");

@ ***************************************************** @
@ Empirical mean, variance and CIs of central forecasts @
@ ***************************************************** @
@ mean: empirical mean: (all nh by mg)
  med: empirical median 
  std: empirical standard deviation 
  L90: 90% lower confidence bound 
  U90: 90% upper confidence bound 
  L95: 95% lower confidence bound 
  U95: 95% upper confidence bound @

{ Lmean, Lmed, Lstd, LL90, LU90, LL95, LU95 }
= EMPDIST(SFL,nhorz,mg,n_stoc);

{ Dmean, Dmed, Dstd, DL90, DU90, DL95, DU95 }
= EMPDIST(SFD,nhorz,mg,n_stoc);

{ A4Lmean, A4Lmed, A4Lstd, A4LL90, A4LU90, A4LL95, A4LU95 }
= EMPDIST(SA4FL,nhorz,mg,n_stoc);

{ A4Dmean, A4Dmed, A4Dstd, A4DL90, A4DU90, A4DL95, A4DU95 }
= EMPDIST(SA4FD,nhorz,mg,n_stoc);

{ ALmean, ALmed, ALstd, ALL90, ALU90, ALL95, ALU95 }
= EMPDIST(SAFL,nhorz,mg,n_stoc);

{ ADmean, ADmed, ADstd, ADL90, ADU90, ADL95, ADU95 }
= EMPDIST(SAFD,nhorz,mg,n_stoc);

@ Exporting the results to Excel file @

SF1 = xlswritem(Lmean,"LFOR","b3",1,"");
SF2 = xlswritem(Lmed, "LFOR","b3",2,"");
SF3 = xlswritem(Lstd, "LFOR","b3",3,"");
SF4 = xlswritem(LL90, "LFOR","b3",4,"");
SF5 = xlswritem(LU90, "LFOR","b3",5,"");
SF6 = xlswritem(LL95, "LFOR","b3",6,"");
SF7 = xlswritem(LU95, "LFOR","b3",7,"");

SF1 = xlswritem(Dmean,"DFOR","b3",1,"");
SF2 = xlswritem(Dmed, "DFOR","b3",2,"");
SF3 = xlswritem(Dstd, "DFOR","b3",3,"");
SF4 = xlswritem(DL90, "DFOR","b3",4,"");
SF5 = xlswritem(DU90, "DFOR","b3",5,"");
SF6 = xlswritem(DL95, "DFOR","b3",6,"");
SF7 = xlswritem(DU95, "DFOR","b3",7,"");

SF1 = xlswritem(A4Lmean,"A4LFOR","b3",1,"");
SF2 = xlswritem(A4Lmed, "A4LFOR","b3",2,"");
SF3 = xlswritem(A4Lstd, "A4LFOR","b3",3,"");
SF4 = xlswritem(A4LL90, "A4LFOR","b3",4,"");
SF5 = xlswritem(A4LU90, "A4LFOR","b3",5,"");
SF6 = xlswritem(A4LL95, "A4LFOR","b3",6,"");
SF7 = xlswritem(A4LU95, "A4LFOR","b3",7,"");

SF1 = xlswritem(A4Dmean,"A4DFOR","b3",1,"");
SF2 = xlswritem(A4Dmed, "A4DFOR","b3",2,"");
SF3 = xlswritem(A4Dstd, "A4DFOR","b3",3,"");
SF4 = xlswritem(A4DL90, "A4DFOR","b3",4,"");
SF5 = xlswritem(A4DU90, "A4DFOR","b3",5,"");
SF6 = xlswritem(A4DL95, "A4DFOR","b3",6,"");
SF7 = xlswritem(A4DU95, "A4DFOR","b3",7,"");

SF1 = xlswritem(ALmean,"ALFOR","b3",1,"");
SF2 = xlswritem(ALmed, "ALFOR","b3",2,"");
SF3 = xlswritem(ALstd, "ALFOR","b3",3,"");
SF4 = xlswritem(ALL90, "ALFOR","b3",4,"");
SF5 = xlswritem(ALU90, "ALFOR","b3",5,"");
SF6 = xlswritem(ALL95, "ALFOR","b3",6,"");
SF7 = xlswritem(ALU95, "ALFOR","b3",7,"");

SF1 = xlswritem(ADmean,"ADFOR","b3",1,"");
SF2 = xlswritem(ADmed, "ADFOR","b3",2,"");
SF3 = xlswritem(ADstd, "ADFOR","b3",3,"");
SF4 = xlswritem(ADL90, "ADFOR","b3",4,"");
SF5 = xlswritem(ADU90, "ADFOR","b3",5,"");
SF6 = xlswritem(ADL95, "ADFOR","b3",6,"");
SF7 = xlswritem(ADU95, "ADFOR","b3",7,"");

sfj = 1;
do until sfj > 7;
    ppp1 = xlswritesa(header, "LFOR",  "b2", sfj,"");
    ppp2 = xlswritesa(header, "DFOR",  "b2", sfj,"");
    ppp3 = xlswritesa(header, "A4LFOR","b2", sfj,"");
    ppp4 = xlswritesa(header, "A4DFOR","b2", sfj,"");
    ppp5 = xlswritesa(header, "ALFOR", "b2", sfj,"");
    ppp6 = xlswritesa(header, "ADFOR", "b2", sfj,"");
    sfj = sfj + 1;
endo;

qq1 = {"Mean"};
qq2 = {"Median"};
qq3 = {"St dev"};
qq4 = {"Lower90%"};
qq5 = {"Upper90%"};
qq6 = {"Lower95%"};
qq7 = {"Upper95%"};

qqq1 = xlswritesa(qq1, "LFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "LFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "LFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "LFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "LFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "LFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "LFOR", "a2", 7,"");

qqq1 = xlswritesa(qq1, "DFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "DFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "DFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "DFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "DFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "DFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "DFOR", "a2", 7,"");

qqq1 = xlswritesa(qq1, "A4LFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "A4LFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "A4LFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "A4LFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "A4LFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "A4LFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "A4LFOR", "a2", 7,"");

qqq1 = xlswritesa(qq1, "A4DFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "A4DFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "A4DFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "A4DFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "A4DFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "A4DFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "A4DFOR", "a2", 7,"");

qqq1 = xlswritesa(qq1, "ALFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "ALFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "ALFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "ALFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "ALFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "ALFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "ALFOR", "a2", 7,"");

qqq1 = xlswritesa(qq1, "ADFOR", "a2", 1,"");
qqq2 = xlswritesa(qq2, "ADFOR", "a2", 2,"");
qqq3 = xlswritesa(qq3, "ADFOR", "a2", 3,"");
qqq4 = xlswritesa(qq4, "ADFOR", "a2", 4,"");
qqq5 = xlswritesa(qq5, "ADFOR", "a2", 5,"");
qqq6 = xlswritesa(qq6, "ADFOR", "a2", 6,"");
qqq7 = xlswritesa(qq7, "ADFOR", "a2", 7,"");

@ Country header @

qs1 = xlswritesa(CHD, "LFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "LFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "LFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "LFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "LFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "LFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "LFOR", "b1",7,"");

qs1 = xlswritesa(CHD, "DFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "DFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "DFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "DFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "DFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "DFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "DFOR", "b1",7,"");

qs1 = xlswritesa(CHD, "A4LFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "A4LFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "A4LFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "A4LFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "A4LFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "A4LFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "A4LFOR", "b1",7,"");

qs1 = xlswritesa(CHD, "A4DFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "A4DFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "A4DFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "A4DFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "A4DFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "A4DFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "A4DFOR", "b1",7,"");

qs1 = xlswritesa(CHD, "ALFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "ALFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "ALFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "ALFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "ALFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "ALFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "ALFOR", "b1",7,"");

qs1 = xlswritesa(CHD, "ADFOR", "b1",1,"");
qs2 = xlswritesa(CHD, "ADFOR", "b1",2,"");
qs3 = xlswritesa(CHD, "ADFOR", "b1",3,"");
qs4 = xlswritesa(CHD, "ADFOR", "b1",4,"");
qs5 = xlswritesa(CHD, "ADFOR", "b1",5,"");
qs6 = xlswritesa(CHD, "ADFOR", "b1",6,"");
qs7 = xlswritesa(CHD, "ADFOR", "b1",7,"");

time;
END;

@* ********** *@
@* PROCEDURES *@
@* ********** *@

/* ****************************** */
   Proc (2) = Elevel(lhat,zd,n,m);
/* ****************************** */
@ Forecasts of 4-quarter MA and average over n @
local a4hat,ahat,zi,zt4,suml,a4tmp,zj,ztmp,obs;

A4hat = zeros(n,m);
obs = rows(zd);
zt4 = zd[obs-3:obs,.];  @ initial 4 by m @
Ahat = zeros(n,m);  suml = zeros(1,m);

zi = 1;
do until zi > n;
   zt4 = zt4[2:4,.]|Lhat[zi,.];

   A4tmp = zeros(1,m);
   zj = 1;
   do until zj > m;

      ztmp = zt4[.,zj];
      A4tmp[zj] = meanc(ztmp);
      zj = zj + 1;
   endo;
   A4hat[zi,.] = A4tmp;
   sumL = sumL + Lhat[zi,.];
   Ahat[zi,.] = sumL./zi;

   zi = zi + 1;
endo;

RETP(a4hat,ahat); ENDP;

/* ******************************* */
   Proc (3) = Echange(lhat,zd,n,m);
/* ******************************* */
local zi,ZT0,zt4,llhat,llhat1,llhat2,llhat3,llhat4,dhat,d4hat,adhat,sumd,obs;
@ Forecasts of the first-differences @
obs = rows(zd);
ZT0 = Zd[obs,.];
LLhat = ZT0|Lhat;
LLhat1 = lag(LLhat);
Dhat = LLhat - LLhat1;
Dhat = Dhat[2:rows(Dhat),.];  @ n by m matrix of forecasts of growth @

@ Forecasts of the fourth-difference divided by 4: annual rate @
zt4 = zd[obs-3:obs,.];  @ ?? z at time T, T-1, T-2, T-3 @
LLhat  = zt4|Lhat;      @ n+4 by m @
LLhat1 = lag(LLhat);
LLhat2 = lag(LLhat1);
LLhat3 = lag(LLhat2);
LLhat4 = lag(LLhat3);
D4hat = LLhat - LLhat4;
D4hat = D4hat[5:rows(D4hat),.];
D4hat = D4hat./4;

@ Forecasts of the averages @
ADhat = zeros(n,m);  sumD = zeros(1,m);
zi = 1;
do until zi > n;
   sumD = sumD + Dhat[zi,.];
   Adhat[zi,.] = sumD./zi;
   zi = zi + 1;
endo;

RETP(dhat,d4hat,adhat);ENDP;

@ ********************************************************* @
  Proc (10) = PREVENT(A4DP,DY,A4DY,DYT,THDP,THDY,EVDP,EVDY);
@ ********************************************************* @
local flag1,flag2,flag3,flag4,flag5,flag6,flag7,flag8,flag9,flag10,
      ndp,ndy,ns,nh,ti0,ti1,tj,hat1,hat2,hat3,hat30;

ndp = rows(thdp); ndy = rows(thdy); @ number of thresholds @

@ transformation to make it n_sto by nh @
A4DP = A4DP'; DY = DY'; A4DY = A4DY';
ns = rows(a4dp); @ number of replications @ 
nh = cols(a4dp); @ number of horizons @

@ conversion to annual % @
A4DP = 400.*A4DP; DY = 400.*DY; A4DY = 400.*A4DY; 

@ E1: 4-qt inflation with 15 thresholds @
@ E3: PR( a% < 4qt MA dp < b%) @
@ E4: PR(4qt MA dp < c%) @

flag1 = zeros(nh,ndp);  @ nhorizon by number of thresholds @
flag3 = zeros(nh,1); 
flag4 = zeros(nh,1);
ti0 = 1;
do until ti0 > nh;
   ti1 = 1;
   do until ti1 > ns;

      hat1 = A4DP[ti1,ti0];
      tj = 1;
      do until tj > ndp;
         if hat1 < thdp[tj]; flag1[ti0,tj]=flag1[ti0,tj]+1;  
         endif;
      tj = tj + 1;
      endo;

      if hat1 > evdp[1]; if hat1 < evdp[2]; flag3[ti0]=flag3[ti0]+1;
                         endif;
      endif;

      if hat1 < evdp[3]; flag4[ti0]=flag4[ti0]+1;
      endif;
      ti1 = ti1+1;
   endo;
   ti0 = ti0+1;
endo;
flag1 = flag1./ns; @ probability @
flag3 = flag3./ns;
flag4 = flag4./ns;

@ E2: 4-qt output growth with 15 thresholds @
@ E5: Recession: Pr(quarterly dy <0 and dy(-1) 0) @
@ E6: Bad growth Pr(4-qt dy <(avg growth/2)%) @

flag2 = zeros(nh,ndp); flag5 = zeros(nh,1); flag6 = zeros(nh,1);
ti0 = 1;
do until ti0 > nh;
   ti1 = 1;
   do until ti1 > ns;

      hat2 = A4DY[ti1,ti0];
      tj = 1;
      do until tj > ndy;
         if hat2 < thdy[tj]; flag2[ti0,tj]=flag2[ti0,tj]+1;  
         endif;
      tj = tj + 1;
      endo;

      if hat2 < evdy[2]; flag6[ti0]=flag6[ti0]+1;
      endif;

      hat3 = DY[ti1,ti0];
      if ti0 eq 1;
         hat30 = dyT;
         if hat3 < evdy[1];
            if hat30 < evdy[1]; flag5[ti0]=flag5[ti0]+1;
            endif;
         endif;
      else;
         hat30 = DY[ti1,ti0-1];
         if hat3 < evdy[1];
            if hat30 < evdy[1]; flag5[ti0]=flag5[ti0]+1;
            endif;
         endif;
      endif;

      ti1 = ti1+1;
   endo;
   ti0 = ti0+1;
endo;
flag2 = flag2./ns; @ probability @
flag5 = flag5./ns;
flag6 = flag6./ns;

@ E7&E9: Joint event of inflation targets 1,2 and no recession @
@ E8&E10: Joint event of inflation targets 1,2 and no bad growth @

flag7 = zeros(nh,1); flag8 = zeros(nh,1);
flag9 = zeros(nh,1); flag10 = zeros(nh,1);
ti0 = 1;
do until ti0 > nh;
   ti1 = 1;
   do until ti1 > ns;

      hat1 = A4DP[ti1,ti0];
      hat2 = A4DY[ti1,ti0];
      hat3 = DY[ti1,ti0];

      if ti0 eq 1; hat30 = dyT;
      else;        hat30 = DY[ti1,ti0-1];
      endif; 

      if hat3 < evdy[1];     @ dy < 0% @
         if hat30 < evdy[1]; @ dy(-1) < 0% @
         else;
            if hat1 > evdp[1];    @ dp > a% @
               if hat1 < evdp[2]; @ dp < b% @
                  flag7[ti0]=flag7[ti0]+1;
               endif;
            endif;

            if hat1 < evdp[3];     @ dp < b% @
               flag9[ti0]=flag9[ti0]+1;
            endif;
         endif;
 
     else;   @ dy > 0% @
         if hat1 > evdp[1];    @ dp > a% @
            if hat1 < evdp[2]; @ dp < b% @
               flag7[ti0]=flag7[ti0]+1;
            endif;
         endif;

         if hat1 < evdp[3];     @ dp < b% @
            flag9[ti0]=flag9[ti0]+1;
         endif;
      endif;

      if hat2 < evdy[2]; @ bad growth @
      else; 
         if hat1 > evdp[1];    @ dp > a% @
            if hat1 < evdp[2]; @ dp < b% @
               flag8[ti0]=flag8[ti0]+1;
            endif;
         endif;

         if hat1 < evdp[3];     @ dp < b% @
            flag10[ti0]=flag10[ti0]+1;
         endif;
     endif;

      ti1 = ti1 + 1;
   endo;
   ti0 = ti0 + 1;
endo;
flag7 = flag7./ns; @ probability @
flag8 = flag8./ns;
flag9 = flag9./ns;
flag10 = flag10./ns;

RETP(flag1,flag2,flag3,flag4,flag5,flag6,flag7,flag8,flag9,flag10);ENDP;

@ ******************************* @
  Proc (1) = PREVCA(A4CA,A4DCA);
@ ******************************* @
local fg,fg1,fg2,fg3,fg4,fg5,fg6,ns,nh,ti0,ti1,hat1,hat2;

@ transformation to make it n_sto by nh @
A4CA = A4CA'; A4DCA = A4DCA';
ns = rows(a4CA); @ number of replications @ 
nh = cols(a4CA); @ number of horizons @

@ conversion to annual % @
A4CA = 400.*A4CA; A4DCA = 400.*A4DCA; 

@ E1. A single event of CA deficit @
@ E2. A single event of CA improvement @
@ E3. A joint event of CA deficit & deterioration @
@ E4. A joint event of CA deficit & improvement  @
@ E5. A joint event of CA surplus & deterioration  @
@ E6. A joint event of CA surplus & improvement @

fg1 = zeros(nh,1); fg2 = zeros(nh,1); fg3 = zeros(nh,1); 
fg4 = zeros(nh,1); fg5 = zeros(nh,1); fg6 = zeros(nh,1); 
ti0 = 1;
do until ti0 > nh;
   ti1 = 1;
   do until ti1 > ns;

      hat1 = A4CA[ti1,ti0];
      hat2 = A4DCA[ti1,ti0];

      if hat1 < 0; fg1[ti0] = fg1[ti0] + 1;     @ CA deficit @
         if hat2 < 0; fg3[ti0] = fg3[ti0] + 1;  @ CA deficit & deterioration @
         else;        fg4[ti0] = fg4[ti0] + 1;  @ CA deficit & improvement @
         endif;
      else;
         if hat2 < 0; fg5[ti0] = fg5[ti0] + 1;  @ CA surplus & deterioration @
         else;        fg6[ti0] = fg6[ti0] + 1;  @ CA surplus & improvement @
         endif;
      endif;

      if hat2 > 0; fg2[ti0] = fg2[ti0] + 1;     @ CA improvement @
      endif;

      ti1 = ti1+1;
   endo;
   ti0 = ti0+1;
endo;
fg = fg1~fg2~fg3~fg4~fg5~fg6;
fg = fg./ns; @ probability @

RETP(fg);ENDP;


@ ********************************* @
  proc (7) = EMPDIST(ForZ,nh,m,ns);
@ ********************************* @
@ Empirical mean, variance and CIs of central forecasts @
@ ***************************************************** @
local nhm,mean,med,std,L90,U90,L95,U95,j,tmp;

nhm = nh*m;
mean = zeros(nhm,1); @ empirical mean: n*m by 1 @
med = zeros(nhm,1);  @ empirical median @
std = zeros(nhm,1);  @ empirical standard deviation @
L90 = zeros(nhm,1);  @ 90% lower confidence bound @
U90 = zeros(nhm,1);  @ 90% upper confidence bound @
L95 = zeros(nhm,1);  @ 95% lower confidence bound @
U95 = zeros(nhm,1);  @ 95% upper confidence bound @

j=1;
do until j > rows(ForZ);
   tmp = ForZ[j,.]';
   { mean[j], med[j], std[j], l90[j], u90[j], l95[j], u95[j] } 
   = EMPD(tmp,ns);
   j=j+1;
endo;

mean = TRMAT(mean,nh,m);
med = TRMAT(med,nh,m);
std = TRMAT(std,nh,m);
L90 = TRMAT(l90,nh,m);
U90 = TRMAT(u90,nh,m);
L95 = TRMAT(l95,nh,m);
U95 = TRMAT(u95,nh,m);

retp(mean,med,std,L90,U90,L95,U95);endp;

@ ************************ @
  proc (7) = EMPD(tmp,n1);
@ ************************ @
local mean,med,std,fr50,l90,u90,l95,u95,stmp;

l90 = n1*0.05;   l90 = ceil(l90);
u90 = n1*0.95;   u90 = floor(u90);
l95 = n1*0.025;  l95 = ceil(l95);
u95 = n1*0.975;  u95 = floor(u95);
fr50 = n1*0.5;   fr50 = round(fr50); 

mean = meanc(tmp);
std = stdc(tmp);

stmp = sortc(tmp,1);
l90 = stmp[l90];
u90 = stmp[u90];
l95 = stmp[l95];
u95 = stmp[u95];
med = stmp[fr50];

retp(mean,med,std,l90,u90,l95,u95);endp;

@ ***************************** @
  proc (1) = TRMAT(vect,ro,co);
@ ***************************** 
* Converting nm by 1 vector to n by m matrix 
* ****************************************** @
local matri,smj,rj,cj;

matri = zeros(ro,co);
smj = 1;
do until smj > co;
   rj = (smj-1)*ro + 1;
   cj = smj*ro;
   matri[.,smj] = vect[rj:cj];
   smj = smj + 1;
endo;

retp(Matri);endp;




